require 'rails/generators/resource_helpers'

class ComposerGenerator < Rails::Generators::NamedBase
  include Rails::Generators::ResourceHelpers
  source_root File.expand_path('templates', __dir__)

  def install
    gen_route if yes?("gen route?")
    gen_model if yes?("gen model?")
    gen_controller if yes?("gen controller?")  
    gen_view if yes?("gen view?")
    gen_pundit if yes?("gen pundit?")
    gen_serializer if yes?("gen serializer?")
  end

  private

  def gen_route
    inject_into_file 'config/routes.rb', after: "# composer flag\n" do <<-RUBY

  resources :#{plural_name}
  resources :#{plural_name} do
    get :foo
    get :bar, on: :collection
  end

    RUBY
    end
  end

  def gen_model
    template 'models/model.rb.erb', File.join("app/models", "#{singular_name}.rb")
  end

  def gen_controller
    if yes?("does it need a namespace?")
      @namespace = ask("pls, input namespace name:")
      @namespace = @namespace.split('::').map(&:camelize).join("::")
      template 'controllers/namespace_controller.rb.erb', File.join("app/controllers", @namespace.underscore, "#{singular_name}.rb")
    else
      template 'controllers/controller.rb.erb', File.join("app/controllers", "#{singular_name}.rb")
    end
  end

  def gen_view
    actions = "index show new edit destory _form _record"
    if yes?("does it need a namespace?")
      @namespace = ask("pls, input namespace name:")
      @namespace = @namespace.split('::').map(&:camelize).join("::")
      actions.split(" ").each do |action|
        case action
        when 'destory'
          template "views/destroy.html.erb", File.join("app/views", @namespace.underscore, "destroy.turbo_stream.haml")
        when '_record'
          filename = "_#{singular_name}.html.haml"
          template "views/_record.html.erb", File.join("app/views", @namespace.underscore, filename)
        else
          template "views/#{action}.html.erb", File.join("app/views", @namespace.underscore, "#{action}.html.haml")
        end
      end
    else
      actions.split(" ").each do |action|
        case action
        when 'destory'
          template "views/destroy.html.erb", File.join("app/views", "destroy.turbo_stream.haml")
        when '_record'
          filename = "_#{singular_name}.html.haml"
          template "views/_record.html.erb", File.join("app/views", filename)
        else
          template "views/#{action}.html.erb", File.join("app/views", "#{action}.html.haml")
        end
      end
    end
  end

  def gen_pundit
    if yes?("does it need a namespace?")
      @namespace = ask("pls, input namespace name:")
      @namespace = @namespace.split('::').map(&:camelize).join("::")
      template 'policies/pundit.rb.erb', File.join("app/policies", @namespace.underscore, "#{singular_name.underscore}_policy.rb")
    else
      template 'policies/pundit.rb.erb', File.join("app/policies", "#{singular_name.underscore}_policy.rb")
    end
  end

  def gen_serializer
    if yes?("does it need a namespace?")
      @namespace = ask("pls, input namespace name:")
      @namespace = @namespace.split('::').map(&:camelize).join("::")
      template 'serializers/namespace_serializer.rb.erb', File.join("app/serializers", @namespace.underscore, "#{singular_name}_serializer.rb")
    else
      template 'serializers/serializer.rb.erb', File.join("app/serializers", "#{singular_name}_serializer.rb")
    end
  end
end